//+------------------------------------------------------------------+
//|                                                TimeSeriesExt.mqh |
//|                          Standard Library Patch for MetaTrader 4 |
//|                                                       28/02/2017 |
//+------------------------------------------------------------------+
#include <Arrays\ArrayInt.mqh>
#include <Arrays\ArrayLong.mqh>

#define DEFAULT_BUFFER_SIZE 1024

//+------------------------------------------------------------------+
//| Class CTickVolumeBuffer.                                         |
//| Purpose: Class of buffer of tick volume series.                  |
//|          Derives from class CArrayLong.                          |
//+------------------------------------------------------------------+
class CTickVolumeBuffer : public CArrayLong
{
  protected:
    string            m_symbol;           // symbol
    ENUM_TIMEFRAMES   m_period;           // period
    int               m_freshed_data;     // number of refreshed data
    int               m_size;             // size of used history

  public:
    CTickVolumeBuffer(void);
    ~CTickVolumeBuffer(void);
    // methods of access to protected data
    void              Size(const int size)
    {
      m_size = size;
    }
    // methods of access to data
    long              At(const int index) const;
    // method of refreshing of the data buffer
    virtual bool      Refresh(void);
    virtual bool      RefreshCurrent(void);
    // methods of tuning
    void              SetSymbolPeriod(const string symbol, const ENUM_TIMEFRAMES period);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CTickVolumeBuffer::CTickVolumeBuffer(void) : m_symbol(""),
  m_period(WRONG_VALUE),
  m_freshed_data(0),
  m_size(DEFAULT_BUFFER_SIZE)
{
  ArraySetAsSeries(m_data, true);
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CTickVolumeBuffer::~CTickVolumeBuffer(void)
{
}
//+------------------------------------------------------------------+
//| Access to data in a position                                     |
//+------------------------------------------------------------------+
long CTickVolumeBuffer::At(const int index) const
{
  // check
  if(index >= m_data_total)
    return(0);

  return((datetime)CArrayLong::At(index));
}
//+------------------------------------------------------------------+
//| Refreshing of data buffer                                        |
//+------------------------------------------------------------------+
bool CTickVolumeBuffer::Refresh(void)
{
  m_freshed_data = CopyTickVolume(m_symbol, m_period, 0, m_size, m_data);

  if(m_freshed_data > 0)
  {
    m_data_total = ArraySize(m_data);
    return(true);
  }
  // error
  return(false);
}
//+------------------------------------------------------------------+
//| Refreshing of data buffer                                        |
//+------------------------------------------------------------------+
bool CTickVolumeBuffer::RefreshCurrent(void)
{
  long array[1];

  if(CopyTickVolume(m_symbol, m_period, 0, 1, array) == 1 && m_data_total > 0)
  {
    m_data[0] = array[0];
    return(true);
  }
  // error
  return(false);
}
//+------------------------------------------------------------------+
//| Set symbol and period                                            |
//+------------------------------------------------------------------+
void CTickVolumeBuffer::SetSymbolPeriod(const string symbol, const ENUM_TIMEFRAMES period)
{
  m_symbol = (symbol == NULL) ? ChartSymbol() : symbol;
  m_period = (period == 0)    ? ChartPeriod() : period;
}

//+------------------------------------------------------------------+
//| Class CSpreadBuffer.                                             |
//| Purpose: Class of buffer of spread series.                       |
//|          Derives from class CArrayInt.                           |
//+------------------------------------------------------------------+
class CSpreadBuffer : public CArrayInt
{
  protected:
    string            m_symbol;           // symbol
    ENUM_TIMEFRAMES   m_period;           // period
    int               m_freshed_data;     // number of refreshed data
    int               m_size;             // size of used history

  public:
    CSpreadBuffer(void);
    ~CSpreadBuffer(void);
    // methods of access to protected data
    void              Size(const int size)
    {
      m_size = size;
    }
    // methods of access to data
    int               At(const int index) const;
    // method of refreshing of the data buffer
    virtual bool      Refresh(void);
    virtual bool      RefreshCurrent(void);
    // methods of tuning
    void              SetSymbolPeriod(const string symbol, const ENUM_TIMEFRAMES period);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSpreadBuffer::CSpreadBuffer(void) : m_symbol(""),
  m_period(WRONG_VALUE),
  m_freshed_data(0),
  m_size(DEFAULT_BUFFER_SIZE)
{
  ArraySetAsSeries(m_data, true);
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSpreadBuffer::~CSpreadBuffer(void)
{
}
//+------------------------------------------------------------------+
//| Access to data on the position                                   |
//+------------------------------------------------------------------+
int CSpreadBuffer::At(const int index) const
{
  // check
  if(index >= m_data_total)
    return(0);

  return(CArrayInt::At(index));
}
//+------------------------------------------------------------------+
//| Refreshing of data buffer                                        |
//+------------------------------------------------------------------+
bool CSpreadBuffer::Refresh(void)
{
  m_freshed_data = CopySpread(m_symbol, m_period, 0, m_size, m_data);

  if(m_freshed_data > 0)
  {
    m_data_total = ArraySize(m_data);
    return(true);
  }
  // error
  return(false);
}
//+------------------------------------------------------------------+
//| Refreshing of the data buffer                                    |
//+------------------------------------------------------------------+
bool CSpreadBuffer::RefreshCurrent(void)
{
  int array[1];

  if(CopySpread(m_symbol, m_period, 0, 1, array) == 1 && m_data_total > 0)
  {
    m_data[0] = array[0];
    return(true);
  }
  // error
  return(false);
}
//+------------------------------------------------------------------+
//| Set symbol and period                                            |
//+------------------------------------------------------------------+
void CSpreadBuffer::SetSymbolPeriod(const string symbol, const ENUM_TIMEFRAMES period)
{
  m_symbol = (symbol == NULL) ? ChartSymbol() : symbol;
  m_period = (period == 0)    ? ChartPeriod() : period;
}

//+------------------------------------------------------------------+
//| Class CiSpread.                                                  |
//| Purpose: Class of spread series.                                 |
//|          Derives from class CSeries.                             |
//+------------------------------------------------------------------+
class CiSpread : public CSeries
{
  public:
    CiSpread(void);
    ~CiSpread(void);
    // method of creation
    bool              Create(const string symbol, const ENUM_TIMEFRAMES period);
    virtual bool      BufferResize(const int size);
    // methods of access to data
    int               GetData(const int index) const;
    int               GetData(const int start_pos, const int count, int &buffer[]) const;
    int               GetData(const datetime start_time, const int count, int &buffer[]) const;
    int               GetData(const datetime start_time, const datetime stop_time, int &buffer[]) const;
    // method of refreshing of the data
    virtual void      Refresh(const int flags = OBJ_ALL_PERIODS);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CiSpread::CiSpread(void)
{
  m_name = "Spread";
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CiSpread::~CiSpread(void)
{
}
//+------------------------------------------------------------------+
//| Creating of the spread series                                    |
//+------------------------------------------------------------------+
bool CiSpread::Create(const string symbol, const ENUM_TIMEFRAMES period)
{
  SetSymbolPeriod(symbol, period);
  return(true);
}
//+------------------------------------------------------------------+
//| Method to access data                                            |
//+------------------------------------------------------------------+
int CiSpread::GetData(const int index) const
{
  return (int)MarketInfo(m_symbol, MODE_SPREAD);
}
//+------------------------------------------------------------------+
//| API access method "Copying the spread buffer by specifying       |
//| start position and number of elements"                           |
//+------------------------------------------------------------------+
int CiSpread::GetData(const int start_pos, const int count, int &buffer[]) const
{
  return(CopySpread(m_symbol, m_period, start_pos, count, buffer));
}
//+------------------------------------------------------------------+
//| API access method "Copying the spread buffer by specifying       |
//| start time and number of elements"                               |
//+------------------------------------------------------------------+
int CiSpread::GetData(const datetime start_time, const int count, int &buffer[]) const
{
  return(CopySpread(m_symbol, m_period, start_time, count, buffer));
}
//+------------------------------------------------------------------+
//| API access method "Copying the spread buffer by specifying       |
//| start and end time"                                              |
//+------------------------------------------------------------------+
int CiSpread::GetData(const datetime start_time, const datetime stop_time, int &buffer[]) const
{
  return(CopySpread(m_symbol, m_period, start_time, stop_time, buffer));
}
//+------------------------------------------------------------------+
//| Set size buffer                                                  |
//+------------------------------------------------------------------+
bool CiSpread::BufferResize(const int size)
{
  return(true);
}
//+------------------------------------------------------------------+
//| Refreshing of data                                               |
//+------------------------------------------------------------------+
void CiSpread::Refresh(const int flags)
{
}

//+------------------------------------------------------------------+
//| Class CiTickVolume.                                              |
//| Purpose: Class of tick volume series.                            |
//|          Derives from class CSeries.                             |
//+------------------------------------------------------------------+
class CiTickVolume : public CSeries
{
  public:
    CiTickVolume(void);
    ~CiTickVolume(void);
    // method of creation
    bool              Create(const string symbol, const ENUM_TIMEFRAMES period);
    virtual bool      BufferResize(const int size);
    // methods of access to data
    long              GetData(const int index) const;
    int               GetData(const int start_pos, const int count, long &buffer[]) const;
    int               GetData(const datetime start_time, const int count, long &buffer[]) const;
    int               GetData(const datetime start_time, const datetime stop_time, long &buffer[]) const;
    // method of refreshing of the data
    virtual void      Refresh(const int flags = OBJ_ALL_PERIODS);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CiTickVolume::CiTickVolume(void)
{
  m_name = "Volume";
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CiTickVolume::~CiTickVolume(void)
{
}
//+------------------------------------------------------------------+
//| Creation of the tick volume series                               |
//+------------------------------------------------------------------+
bool CiTickVolume::Create(const string symbol, const ENUM_TIMEFRAMES period)
{
  SetSymbolPeriod(symbol, period);
  return(true);
}
//+------------------------------------------------------------------+
//| Method to access data                                            |
//+------------------------------------------------------------------+
long CiTickVolume::GetData(const int index) const
{
  return iVolume(m_symbol, m_period, index);
}
//+------------------------------------------------------------------+
//| API access method "Copying the tick volume buffer by specifying  |
//| start position and number of elements"                           |
//+------------------------------------------------------------------+
int CiTickVolume::GetData(const int start_pos, const int count, long &buffer[]) const
{
  return(CopyTickVolume(m_symbol, m_period, start_pos, count, buffer));
}
//+------------------------------------------------------------------+
//| API access method "Copying the tick volume buffer by specifying  |
//| start time and number of elements"                               |
//+------------------------------------------------------------------+
int CiTickVolume::GetData(const datetime start_time, const int count, long &buffer[]) const
{
  return(CopyTickVolume(m_symbol, m_period, start_time, count, buffer));
}
//+------------------------------------------------------------------+
//| API access method "Copying the tick volume buffer by specifying  |
//| start and end time"                                              |
//+------------------------------------------------------------------+
int CiTickVolume::GetData(const datetime start_time, const datetime stop_time, long &buffer[]) const
{
  return(CopyTickVolume(m_symbol, m_period, start_time, stop_time, buffer));
}
//+------------------------------------------------------------------+
//| Set size buffer                                                  |
//+------------------------------------------------------------------+
bool CiTickVolume::BufferResize(const int size)
{
  return(true);
}
//+------------------------------------------------------------------+
//| Refreshing of data                                               |
//+------------------------------------------------------------------+
void CiTickVolume::Refresh(const int flags)
{
}

//+------------------------------------------------------------------+
//| Class CRealVolumeBuffer.                                         |
//| Purpose: Class of buffer of real volume series.                  |
//|          Derives from class CArrayLong.                          |
//+------------------------------------------------------------------+
class CRealVolumeBuffer : public CArrayLong
{
  protected:
    string            m_symbol;           // symbol
    ENUM_TIMEFRAMES   m_period;           // period
    int               m_freshed_data;     // number of refreshed data
    int               m_size;             // size of used history

  public:
    CRealVolumeBuffer(void);
    ~CRealVolumeBuffer(void);
    // methods of access to protected data
    void              Size(const int size)
    {
      m_size = size;
    }
    // methods of access to data
    long              At(const int index) const;
    // method of refreshing of the data buffer
    virtual bool      Refresh(void);
    virtual bool      RefreshCurrent(void);
    // methods of tuning
    void              SetSymbolPeriod(const string symbol, const ENUM_TIMEFRAMES period);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CRealVolumeBuffer::CRealVolumeBuffer(void) : m_symbol(""),
  m_period(WRONG_VALUE),
  m_freshed_data(0),
  m_size(DEFAULT_BUFFER_SIZE)
{
  ArraySetAsSeries(m_data, true);
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CRealVolumeBuffer::~CRealVolumeBuffer(void)
{
}
//+------------------------------------------------------------------+
//| Access to data in a position                                     |
//+------------------------------------------------------------------+
long CRealVolumeBuffer::At(const int index) const
{
  // check
  if(index >= m_data_total)
    return(0);

  return((datetime)CArrayLong::At(index));
}
//+------------------------------------------------------------------+
//| Refreshing of data buffer                                        |
//+------------------------------------------------------------------+
bool CRealVolumeBuffer::Refresh(void)
{
  m_freshed_data = CopyRealVolume(m_symbol, m_period, 0, m_size, m_data);

  if(m_freshed_data > 0)
  {
    m_data_total = ArraySize(m_data);
    return(true);
  }
  // error
  return(false);
}
//+------------------------------------------------------------------+
//| Refreshing of data buffer                                        |
//+------------------------------------------------------------------+
bool CRealVolumeBuffer::RefreshCurrent(void)
{
  long array[1];

  if(CopyRealVolume(m_symbol, m_period, 0, 1, array) == 1 && m_data_total > 0)
  {
    m_data[0] = array[0];
    return(true);
  }
  // error
  return(false);
}
//+------------------------------------------------------------------+
//| Set symbol and period                                            |
//+------------------------------------------------------------------+
void CRealVolumeBuffer::SetSymbolPeriod(const string symbol, const ENUM_TIMEFRAMES period)
{
  m_symbol = (symbol == NULL) ? ChartSymbol() : symbol;
  m_period = (period == 0)    ? ChartPeriod() : period;
}
//+------------------------------------------------------------------+
//| Class CiRealVolume.                                              |
//| Purpose: Class of real volume series.                            |
//|          Derives from class CSeries.                             |
//+------------------------------------------------------------------+
class CiRealVolume : public CSeries
{
  public:
    CiRealVolume(void);
    ~CiRealVolume(void);
    // method of creation
    bool              Create(const string symbol, const ENUM_TIMEFRAMES period);
    virtual bool      BufferResize(const int size);
    // methods of access to data
    long              GetData(const int index) const;
    int               GetData(const int start_pos, const int count, long &buffer[]) const;
    int               GetData(const datetime start_time, const int count, long &buffer[]) const;
    int               GetData(const datetime start_time, const datetime stop_time, long &buffer[]) const;
    // method of refreshing of the data
    virtual void      Refresh(const int flags = OBJ_ALL_PERIODS);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CiRealVolume::CiRealVolume(void)
{
  m_name = "RealVolume";
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CiRealVolume::~CiRealVolume(void)
{
}
//+------------------------------------------------------------------+
//| Creation of the real volume series                               |
//+------------------------------------------------------------------+
bool CiRealVolume::Create(const string symbol, const ENUM_TIMEFRAMES period)
{
  SetSymbolPeriod(symbol, period);
  return(false);
}
//+------------------------------------------------------------------+
//| Method to access data                                            |
//+------------------------------------------------------------------+
long CiRealVolume::GetData(const int index) const
{
  return 0;
}
//+------------------------------------------------------------------+
//| API access method "Copying the real volume buffer by specifying  |
//| start position and number of elements"                           |
//+------------------------------------------------------------------+
int CiRealVolume::GetData(const int start_pos, const int count, long &buffer[]) const
{
  return(CopyRealVolume(m_symbol, m_period, start_pos, count, buffer));
}
//+------------------------------------------------------------------+
//| API access method "Copying the real volume buffer by specifying  |
//| start time and number of elements"                               |
//+------------------------------------------------------------------+
int CiRealVolume::GetData(const datetime start_time, const int count, long &buffer[]) const
{
  return(CopyRealVolume(m_symbol, m_period, start_time, count, buffer));
}
//+------------------------------------------------------------------+
//| API access method "Copying the real volume buffer by specifying  |
//| start and end time"                                              |
//+------------------------------------------------------------------+
int CiRealVolume::GetData(const datetime start_time, const datetime stop_time, long &buffer[]) const
{
  return(CopyRealVolume(m_symbol, m_period, start_time, stop_time, buffer));
}
//+------------------------------------------------------------------+
//| Set size buffer                                                  |
//+------------------------------------------------------------------+
bool CiRealVolume::BufferResize(const int size)
{
  return(false);
}
//+------------------------------------------------------------------+
//| Refreshing of data                                               |
//+------------------------------------------------------------------+
void CiRealVolume::Refresh(const int flags)
{
}
//+------------------------------------------------------------------+
